#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include "xutil.h"
#include "lutil.h"
#include "ftn.h"
#include "ftnmsg.h"
#include "rfcmsg.h"
#include "config.h"
#include "bwrite.h"
#include "falists.h"
#include "version.h"
#include "hash.h"

#define MAXHDRSIZE 2048
#define MAXSEEN 70
#define MAXPATH 73

extern time_t now;
extern int newsmode;
extern char *replyaddr;
extern faddr *bestaka;

#ifdef HAS_NDBM_H
extern char *idlookup(char *);
#endif
extern unsigned INT32 crc32(char*,int);
extern char *bgets(char *,int,FILE *);
extern char *strip_flags(char *);
extern void putbody(int,rfcmsg*,FILE*,FILE*,int,int);

static int removemime;
static int removeorg;
static int removeref;
static int ftnorigin;

static char *months[] = {
     "Jan","Feb","Mar","Apr","May","Jun",
     "Jul","Aug","Sep","Oct","Nov","Dec"};

static char *viadate(void);
static char *viadate(void)
{
     static char buf[32];
     time_t t;
     struct tm *ptm;
     time(&t);
     ptm=localtime(&t);
     sprintf(buf,"%d %s %d %02d:%02d:%02d", ptm->tm_mday, months[ptm->tm_mon],
	     ptm->tm_year+1900, ptm->tm_hour,ptm->tm_min,ptm->tm_sec);
     return buf;
}

int putmessage(msg,fmsg,fp,route,flavor,sbl)
rfcmsg *msg;
ftnmsg *fmsg;
FILE *fp;
faddr *route;
char flavor;
fa_list **sbl;
{
	char buf[BUFSIZ], *p,*q,newsubj[MAXSUBJ+1],*oldsubj;
	rfcmsg *tmp;
	int needsplit,hdrsize,datasize,splitpart,forbidsplit;
	int tinyorigin=0;
	faddr *ta;
	fa_list *tmpl;
	fa_list *ptl=NULL;
	int seenlen;
	int oldnet;
	int i;
	char sbe[16];
	FILE *pkt;
	if test_verbose(3) {
	     debug(3,"putmessage from %s",ascfnode(fmsg->from,0x7f));	     
	     debug(3,"putmessage   to %s",ascfnode(fmsg->to,0x7f));
	     debug(3,"putmessage subj %s",S(fmsg->subj));
	     debug(3,"putmessage flags %04x",fmsg->flags);
	     debug(3,"putmessage msgid %s %lu",S(fmsg->msgid_a),fmsg->msgid_n);
	     debug(3,"putmessage reply %s %lu",S(fmsg->reply_a),fmsg->reply_n);
	     if(fmsg->isftn==1)
		  debug(3,"putmessage replytoaddr %s",fmsg->rfcfrom);
	     debug(3,"putmessage date %s",ftndate(fmsg->date));
	}

	removemime=0;
	removeorg=0;
	removeref=0;
	ftnorigin=fmsg->ftnorigin;

	if ((p=hdr("References",msg)))
	{
		p=xstrcpy(p);
		q=strtok(p," \t\n");
		if ((q) &&
		    (strtok(NULL," \t\n") == NULL) &&
		    ((ta=parsefaddr(q))))
		{
			if (ta->name &&
			    (strspn(ta->name,"0123456789") ==
			     strlen(ta->name)))
				removeref=1;
			tidy_faddr(ta);
		}
		free(p);
	}

	p=ascfnode(fmsg->from,0x1f);
	i=79-11-3-strlen(p);
	if (ftnorigin && fmsg->origin && (strlen(fmsg->origin) > i))
	{
		p=ascfnode(fmsg->from,0x0f);
		i=79-11-3-strlen(p);
		tinyorigin=1;
	}
	if (fmsg->origin)
	{
		if (strlen(fmsg->origin) > i)
			fmsg->origin[i]='\0';
		else
			removeorg=1;
	}
	forbidsplit=(ftnorigin || (((p=hdr("X-FTN-Split",msg))) &&
			(strcasecmp(p," already\n") == 0)));
	needsplit=0;
	splitpart=0;
	hdrsize=20;
	hdrsize+=(fmsg->subj)?strlen(fmsg->subj):0;
	hdrsize+=(fmsg->from->name)?strlen(fmsg->from->name):0;
	hdrsize+=(fmsg->to->name)?strlen(fmsg->to->name):0;
	do
	{
		datasize=0;

		if (splitpart)
		{
			sprintf(newsubj,"[part %d] ",splitpart+1);
			strncat(newsubj,fmsg->subj,MAXSUBJ-strlen(newsubj));
		}
		else
		{
			strncpy(newsubj,fmsg->subj,MAXSUBJ);
		}
		newsubj[MAXSUBJ]='\0';

		if (splitpart) hash_update_n(&fmsg->msgid_n,splitpart);

		oldsubj=fmsg->subj;
		fmsg->subj=newsubj;
		if ((pkt=ftnmsghdr(fmsg,NULL,route,flavor)) == NULL)
		{
			return 1;
		}
		fmsg->subj=oldsubj;

		if ((p=hdr("X-FTN-REPLYADDR",msg)))
		{
			hdrsize += 10+strlen(p);
			fprintf(pkt,"\1REPLYADDR:");
			kwrite(p,pkt);
			kwrite("\r",pkt);
		}
		else if (replyaddr)
		{
		     hdrsize += 10+strlen(replyaddr);
		     fprintf(pkt,"\1REPLYADDR: ");
		     kwrite(replyaddr,pkt);
		}

		if ((p=hdr("X-FTN-REPLYTO",msg)))
		{
			hdrsize += 8+strlen(p);
			fprintf(pkt,"\1REPLYTO:");
			kwrite(p,pkt);
			kwrite("\r",pkt);
		}
		else if (replyaddr)
		{
			hdrsize += 15;
			if(fmsg->isftn!=0)
			     fprintf(pkt,"\1REPLYTO: %s %s\r",
				     ascfnode(bestaka,0x1f), "UUCP");
			else
			     fprintf(pkt,"\1REPLYTO: %s\r",
				     ascfnode(bestaka,0x1f));
		}

		if ((p=strip_flags(hdr("X-FTN-FLAGS",msg))))
		{
			hdrsize += 15;
			fprintf(pkt,"\1FLAGS:%s\r",p);
			free(p);
		}

		if ((splitpart == 0) || (hdrsize < MAXHDRSIZE))
		{
			for (tmp=msg;tmp;tmp=tmp->next) 
			     if (!strncmp(tmp->key,"X-FTN-",6) &&
				 strcasecmp(tmp->key,"X-FTN-Tearline") &&
				 strcasecmp(tmp->key,"X-FTN-Origin") &&
				 strcasecmp(tmp->key,"X-FTN-Sender") &&
				 strcasecmp(tmp->key,"X-FTN-Split") &&
				 strcasecmp(tmp->key,"X-FTN-FLAGS") &&
				 strcasecmp(tmp->key,"X-FTN-AREA") &&
				 strcasecmp(tmp->key,"X-FTN-MSGID") &&
				 strcasecmp(tmp->key,"X-FTN-REPLY") &&
				 strcasecmp(tmp->key,"X-FTN-SEEN-BY") &&
				 strcasecmp(tmp->key,"X-FTN-PATH") &&
				 strcasecmp(tmp->key,"X-FTN-REPLYADDR") &&
				 strcasecmp(tmp->key,"X-FTN-REPLYTO") &&
				 strcasecmp(tmp->key,"X-FTN-Via"))
			     {
				  debug(9, "key: %s",tmp->key);
				  if (strcasecmp(tmp->key,"X-FTN-KLUDGE") == 0)
				  {
				       hdrsize += strlen(tmp->val);
				       fprintf(pkt,"\1");
				       kwrite(tmp->val,pkt);
				       kwrite("\r",pkt);
				  }
				  else
				  {
				       hdrsize += strlen(tmp->key)+strlen(tmp->val);
				       fprintf(pkt,"\1%s:",tmp->key+6);
				       kwrite(tmp->val,pkt);
				  }
			     }
			if(usetid==1)			     
			     fprintf(pkt,"\1TID: IFgate %s\r",version);
			if(fmsg->isftn==1)
			{
			     fprintf(pkt,"From: %s\r",fmsg->rfcfrom);
			cwrite("\n",pkt);
			}
		}		
		if (replyaddr)
		{
			free(replyaddr);
			replyaddr=NULL;
		}

		if (needsplit)
		{
			fprintf(pkt," * Continuation %d of a split message *\r\r",
				splitpart);
			needsplit=0;
		}
		else if ((p=hdr("X-Body-Start",msg))) 
		{
			datasize += strlen(p);
			cwrite(p,pkt);
		}
		while (!(needsplit=(!forbidsplit) &&
			 (((splitpart &&
			    (datasize > maxmsize)) ||
			   (!splitpart &&
			    ((datasize+hdrsize) > maxmsize))))) &&
		       (bgets(buf, sizeof(buf)-1, fp)))
		{
		     debug(19,"putmessage body %s",buf);
		     datasize += strlen(buf);
		     cwrite(buf,pkt);
		}
		if (needsplit)
		{
		     fprintf(pkt,"\r * Message split, to be continued *\r");
		     splitpart++;
		}
		
/*		putbody(0,msg,fp,pkt,forbidsplit,hdrsize); */

		fprintf(pkt,"--- ");
		
		if ((p=hdr("X-FTN-Tearline",msg)))
		{
		     if (strcasecmp(p," (none)\n") == 0)
			  cwrite("\n",pkt);
		     else{			  
			  cwrite(p,pkt);
			  cwrite("\n",pkt);			  
		     }
		}
		else
		     if(usereadername==1)
		     {
			  if((p=hdr("X-Newsreader",msg)) ||
			     (p=hdr("X-Mailer",msg)) ||
			     (p=hdr("User-Agent",msg)))
			  {
			       if(strcasecmp(p," (none)\n") == 0)
				    cwrite("\n",pkt);
			       else
			       {
				    cwrite(p,pkt);
				    cwrite("\n",pkt);
			       }
			  }
			  else
			  {
				fprintf(pkt,"IFgate %s (%s)\n",version,osname);
				cwrite("\n",pkt);
			}
		     }
		     else
		    	{
				fprintf(pkt,"IFgate %s (%s)\n",version,osname);
				cwrite("\n",pkt);
			}
		if ((p=hdr("X-FTN-Origin",msg)))
		{
			if (*(q=p+strlen(p)-1) == '\n') *q='\0';
			fprintf(pkt," * Origin:");
			cwrite(p,pkt);
			if (!newsmode) fprintf(pkt,"\r");
		}
		else if (newsmode)
		{
		     fprintf(pkt," * Origin: "); /* strlen=11 */
		     if (fmsg->origin)
			  cwrite(fmsg->origin,pkt);
		     else
			  cwrite("Unknown",pkt);
		     fprintf(pkt," (%s)",
			     ascfnode(fmsg->from,tinyorigin?0x0f:0x1f));
		}

		if (newsmode)
		{
			for (tmpl=whoami;tmpl;tmpl=tmpl->next)
			if ((tmpl->addr->point == 0) &&
			    ((bestaka->domain == NULL) ||
			     (tmpl->addr->domain == NULL) ||
			     (strcasecmp(bestaka->domain,
					tmpl->addr->domain) == 0)) &&
			    (bestaka->zone == tmpl->addr->zone))
				fill_list(sbl,ascfnode(tmpl->addr,0x06),NULL);
#ifdef HAS_NDBM_H
			if ((p=hdr("Message-ID",msg)))
			{
				while (isspace(*p)) p++;
				q=xstrcpy(p);
				if (*(p=q+strlen(q)-1) == '\n') *(p--)='\0';
				while (isspace(*p)) *(p--)='\0';
				fill_list(sbl,idlookup(q),NULL);
				free(q);
			}
#endif
			sort_list(sbl);
			seenlen=MAXSEEN+1;
			/* ensure it will not match for the first entry */
			oldnet=(*sbl)->addr->net-1;
			for (tmpl=*sbl;tmpl;tmpl=tmpl->next)
			{
				if (tmpl->addr->net == oldnet)
					sprintf(sbe," %u",tmpl->addr->node);
				else
					sprintf(sbe," %u/%u",tmpl->addr->net,
							tmpl->addr->node);
				oldnet=tmpl->addr->net;
				seenlen+=strlen(sbe);
				if (seenlen > MAXSEEN)
				{
					seenlen=0;
					fprintf(pkt,"\rSEEN-BY:");
					sprintf(sbe," %u/%u",tmpl->addr->net,
							tmpl->addr->node);
					seenlen=strlen(sbe);
				}
				fprintf(pkt,"%s",sbe);
			}

			for (tmp=msg;tmp;tmp=tmp->next) 
			if (!strcasecmp(tmp->key,"X-FTN-PATH"))
			{
				fill_path(&ptl,tmp->val);
			}
			sprintf(sbe,"%u/%u",bestaka->net,
				bestaka->node);
			fill_path(&ptl,sbe);
			uniq_list(&ptl);
			seenlen=MAXPATH+1;
			/* ensure it will not match for the first entry */
			oldnet=ptl->addr->net-1;
			for (tmpl=ptl;tmpl;tmpl=tmpl->next)
			{
				if (tmpl->addr->net == oldnet)
					sprintf(sbe," %u",tmpl->addr->node);
				else
					sprintf(sbe," %u/%u",tmpl->addr->net,
							tmpl->addr->node);
				oldnet=tmpl->addr->net;
				seenlen+=strlen(sbe);
				if (seenlen > MAXPATH)
				{
					seenlen=0;
					fprintf(pkt,"\r\1PATH:");
					sprintf(sbe," %u/%u",tmpl->addr->net,
							tmpl->addr->node);
					seenlen=strlen(sbe);
				}
				fprintf(pkt,"%s",sbe);
			}
			fprintf(pkt,"\r");
		}
		else /* mail mode */
		{
		     for (tmp=msg;tmp;tmp=tmp->next) 
			  if (!strcasecmp(tmp->key,"X-FTN-Via"))
			  {
			       /*datasize += strlen(tmp->key)+strlen(tmp->val);*/
			       fprintf(pkt,"\1Via");
			       kwrite(tmp->val,pkt);
			  }
		     fprintf(pkt,"\1Via %s IFgate %s %s\r",
			     ascfnode(bestaka,0x1f),
			     version,viadate());
		}
		awrite("",pkt); /* trailing zero byte */
		if (ferror(pkt))
		{
			logerr("$error writing to ftn packet");
			return 1;
		}
		tidy_falist(&ptl);
	}
	while (needsplit);

	debug(3,"putmessage exiting...");
	return 0;
}
