#define PROT_INBOUND "/fido/incoming"
#define FILE_CONFIG "/usr/etc/qico.conf"
#define LOG_PREFIX_LEN 16

#define xfree(p) do { if(p) free(p); p = NULL; } while(0)
#define SAFE(s) s?s:nothing
#define SIZES(x) (((x)<1024)?(x):((x)/1024))
#define SIZEC(x) (((x)<1024)?'b':'k')

#include <fx.h>
#include <fxkeys.h>
#include <stdio.h>
#include <errno.h>
#include <signal.h>
#include <ctype.h>
#include <stdarg.h>
#include <sys/ipc.h>
#include <sys/time.h>
#include <sys/unistd.h>
#include <sys/msg.h>
#include "byteop.h"
#include "config.h"
#include "qcconst.h"
#include "xstr.h"
#include "xmem.h"

#define F_ERR  0
#define F_NORM 1
#define F_HOLD 2
#define F_DIR  3
#define F_CRSH 4
#define F_IMM  5
#define F_REQ  6
#define F_MAX  F_REQ

int sendcommand(int command);


typedef struct _flist_t {
	struct _flist_t *next;
	char *tosend, *sendas, kill;
	FILE *lo;
	off_t loff;
	int type;
} flist_t;


typedef struct {
	int tryes, flags;
	time_t htime, utime;
} sts_t;

typedef struct {
	short int z,n,f,p;
} ftnaddr_t;

typedef struct {
	char sign[20];
	char nlname[MAX_NODELIST][20];
	time_t nltime[MAX_NODELIST];
} idxh_t;

typedef struct {
	ftnaddr_t addr;
	unsigned long offset:24;
	unsigned char index:8;
} idxent_t;

typedef struct _falist_t {
	ftnaddr_t addr;
	struct _falist_t *next;
} falist_t;

typedef struct _slist_t {
	char *str;
	struct _slist_t *next;
} slist_t;

typedef struct _faslist_t {
	ftnaddr_t addr;
	char *str;
	struct _faslist_t *next;
} faslist_t;

typedef struct {
    falist_t *addrs;
	char *name, *place, *sysop, *phone, *wtime, *flags, *pwd, *mailer;
    int options, speed, realspeed, netmail, files, haswtime, hidnum,
		type, holded;
	long int time, starttime;
	char *tty;
} ninfo_t;

typedef struct _cfgitem_t {
	char *condition;
	union {
		int v_int;
		char *v_char;
		falist_t *v_al;
		faslist_t *v_fasl;
		slist_t *v_sl;
	} value;
	struct _cfgitem_t *next;
} cfgitem_t;

typedef struct {
	char *keyword;
	int type, required, found;	
	cfgitem_t *items;
	char *def_val;
} cfgstr_t;

typedef struct _qitem_t {
	ftnaddr_t addr;
	int tryes, flv, what, touched;
	off_t sizes[F_MAX+1];
	time_t times[F_MAX+1];
	off_t reqs, pkts;
	time_t onhold;
	struct _qitem_t *next;
} qitem_t;

typedef struct _dialine_t {
	char *phone, *timegaps;
	int num;
	struct _dialine_t *next;
} dialine_t;

typedef struct _subst {
	ftnaddr_t addr;
	dialine_t *hiddens, *current;
	int nhids;
	struct _subst *next;
} subst_t;

#define CFG_PROTORDER		0
#define CFG_MAILONLY		1
#define CFG_ZMH		2
#define CFG_FREQTIME		3
#define CFG_STATION		4
#define CFG_PLACE		5
#define CFG_SYSOP		6
#define CFG_WORKTIME		7
#define CFG_FLAGS		8
#define CFG_PHONE		9
#define CFG_SPEED		10
#define CFG_ADDRESS		11
#define CFG_NLPATH		12
#define CFG_NODELIST		13
#define CFG_OUTBOUND		14
#define CFG_LOG		15
#define CFG_MASTERLOG		16
#define CFG_PORT		17
#define CFG_DIALDELAY		18
#define CFG_DIALDELTA		19
#define CFG_RESCANPERIOD		20
#define CFG_SUBST		21
#define CFG_PHONETR		22
#define CFG_CANCALL		23
#define CFG_MODEMOK		24
#define CFG_MODEMCONNECT		25
#define CFG_MODEMERROR		26
#define CFG_MODEMBUSY		27
#define CFG_MODEMRESET		28
#define CFG_MODEMHANGUP		29
#define CFG_DIALPREFIX		30
#define CFG_DIALSUFFIX		31
#define CFG_MAX_FAILS		32
#define CFG_WAITRESET		33
#define CFG_WAITCARRIER		34
#define CFG_PASSWORD		35
#define CFG_INBOUND		36
#define CFG_NODIAL		37
#define CFG_SKIPFILES		38
#define CFG_SHOWINTRO		39
#define CFG_MAXSESSION		40
#define CFG_WAITHRQ		41
#define CFG_HOLDONSUCCESS		42
#define CFG_MODEMRINGING		43
#define CFG_MAXRINGS		44
#define CFG_EXTRP		45
#define CFG_FREQFROM		46
#define CFG_FREQSUBJ		47
#define CFG_HISTORY		48
#define CFG_FILEBOX		49
#define CFG_LONGBOXPATH		50
#define CFG_MAPPATH		51
#define CFG_MAPOUT		52
#define CFG_MAPIN		53
#define CFG_MINSPEED		54
#define CFG_AUTOSKIP		55
#define CFG_AFTERSESSION		56
#define CFG_EMSILOG		57
#define CFG_PIDFILE		58
#define CFG_DEFPERM		59
#define CFG_DIRPERM		60
#define CFG_IPCPERM		61
#define CFG_PROGNAME		62
#define CFG_VERSION		63
#define CFG_OSNAME		64
#define CFG_MINCPSIN		65
#define CFG_MINCPSOUT		66
#define CFG_MINCPSDELAY		67
#define CFG_LOCKDIR		68
#define CFG_CLEARUNDIAL		69
#define CFG_AFTERMAIL		70
#define CFG_CALLONFLAVORS		71
#define CFG_IMMONFLAVORS		72
#define CFG_POLLFLAVOR		73
#define CFG_ZRXWIN		74
#define CFG_ZTXWIN		75
#define CFG_HRXWIN		76
#define CFG_HTXWIN		77
#define CFG_JRXWIN		78
#define CFG_JTXWIN		79
#define CFG_STANDARDEMSI		80
#define CFG_ESTIMATEDTIME		81
#define CFG_LOGLEVELS		82
#define CFG_ROOTDIR		83
#define CFG_ALWAYSKILLFILES		84
#define CFG_NEEDALLLISTED		85

#define CFG_NNN		86


extern cfgstr_t configtab[CFG_NNN+1];

extern "C" {char *bso_tmp;
#undef word
#undef byte
#undef dword
#define dword unsigned long
#define word unsigned short
#define byte unsigned char
  extern FILE *txfd;
  extern FILE *rxfd;  
  extern long txpos;
  extern long rxpos;
  extern word txretries;
  extern word rxretries;   
  extern long txwindow;
  extern long rxwindow;     
  extern word txblklen;
  extern word rxblklen;
  extern long txsyncid;
  extern long rxsyncid;    
  extern byte *txbuf;
  extern byte *rxbuf;
  extern dword txoptions;
  extern dword rxoptions;  
  extern unsigned effbaud;
  extern byte *rxbufptr;
  extern byte txstate;
  extern byte rxstate;
  extern byte *rxbufmax;
  extern long txstart;
  extern long rxstart;  
  extern word txmaxblklen;
  extern word timeout;
  extern byte txlastc;
  extern int bso_init(char *bsopath, int def_zone);
  extern void bso_done(void);
  extern char *bso_name(ftnaddr_t *fa);
  extern int bso_rescan(void (*each)(char *, ftnaddr_t *, int, int));
  extern int bso_flavor(char fl);
  extern char *bso_pktn(ftnaddr_t *fa, int fl);
  extern char *bso_flon(ftnaddr_t *fa, int fl);
  extern char *bso_bsyn(ftnaddr_t *fa);
  extern char *bso_reqn(ftnaddr_t *fa);
  extern char *bso_stsn(ftnaddr_t *fa);
  extern int bso_locknode(ftnaddr_t *adr);
  extern int bso_unlocknode(ftnaddr_t *adr);
  extern int bso_attach(ftnaddr_t *adr, int flv, slist_t *files);
  extern int bso_request(ftnaddr_t *adr, slist_t *files);
  extern int bso_rmstatus(ftnaddr_t *adr);
  extern int bso_setstatus(ftnaddr_t *fa, sts_t *st);
  extern int bso_getstatus(ftnaddr_t *fa, sts_t *st);
  extern int bso_poll(ftnaddr_t *fa, int flavor);
  extern int parseftnaddr(char *s, ftnaddr_t *a, ftnaddr_t *b, int wc);
  extern char *ftnaddrtoa(ftnaddr_t *a);
  extern slist_t *slist_add(slist_t **l, char *s);
  extern void slist_kill(slist_t **l);
  extern ftnaddr_t DEFADDR;
  //
  extern int cci;
  extern char *ccs;
  extern slist_t *ccsl;
  extern faslist_t *ccfasl;
  extern falist_t *ccal;
  extern int getstr(char **to, char *from);
  extern int getpath(char **to, char *from);
  extern int getlong(int *to, char *from);
  extern int getaddrl(falist_t **to, char *from);
  extern int getfasl(faslist_t **to, char *from);
  extern int getyesno(int *to, char *from);
  extern int getstrl(slist_t **to, char *from);
  extern int setvalue(cfgitem_t *ci, char *t, int type);
  extern int cfgi(int i);
  extern char *cfgs(int i);
  extern slist_t *cfgsl(int i);
  extern faslist_t *cfgfasl(int i);
  extern falist_t *cfgal(int i);
  extern int readconfig(char *cfgname);
  extern int parseconfig(char *cfgname);
  extern void killconfig(void);
#ifdef NEED_DEBUG
  extern void dumpconfig();
#endif
  //
  extern qitem_t *q_queue;
  extern qitem_t *q_find(ftnaddr_t *fa);
  extern int q_rescan(qitem_t **curr);
  extern off_t q_sum(qitem_t *q);
  extern void qsendqueue();
  //
  extern int qipc_init();
  extern void qipc_done();
  extern void vlogs(char *str);
  extern void vlog(char *str, ...);
  extern void sline(char *str, ...);
  extern void title(char *str, ...);
  extern void qsendpkt(char what, const char *line, char *buff, int len);
  extern void qpreset(int snd);
  extern void qemsisend(ninfo_t *e);
  extern void qpqueue(ftnaddr_t *a, int mail, int files, int tryes, int flags);
  extern void qpproto(char type, pfile_t *pf);
  extern void simulate_send(ftnaddr_t *fa);
  extern char *log_tty;
#define t_set(expire) (time(NULL)+expire)
  /* globals.c */
  extern char *devnull;
  extern char *osname;
  extern int is_ip;
  extern int do_rescan;
  extern pfile_t sendf;
  extern pfile_t recvf;
  extern long int temptot;
  extern long int temptot2;
  extern char ip_id[10];
  extern ninfo_t *rnode;
  extern ftnaddr_t DEFADDR;
  extern unsigned long int totalf, totalm, totaln;
  extern int was_req, got_req;
  extern flist_t *fl;

  //
#define QLNAME is_ip?ip_id:(log_tty?log_tty:"master")
  //
  extern char *NL_SIGN;
  extern char *NL_IDX;
  extern int query_nodelist(ftnaddr_t *addr, char *nlpath, ninfo_t **nl);
  extern int is_listed(falist_t *addr, char *nlpath, int needall);
  extern void phonetrans(char **pph, slist_t *phtr);
  extern int checktimegaps(char *ranges);
  extern int chktxy(char *p);
  extern int checktxy(char *flags);
  extern subst_t *findsubst(ftnaddr_t *fa, subst_t *subs);
  extern subst_t *parsesubsts(faslist_t *sbs);
  extern int applysubst(ninfo_t *nl, subst_t *subs);
  extern void killsubsts(subst_t **l);
  extern int can_dial(ninfo_t *nl, int ct);
  extern int find_dialable_subst(ninfo_t *nl, int ct, subst_t *subs);
  extern void nlfree(ninfo_t *nl);
  extern void nlkill(ninfo_t **nl);
}

FXColor              back_color;     // Colors
FXColor              text_color;
FXColor              hilite_color;
FXFont*              font;           // Font
FXHiliteStyle*       hilitestyles;
int flag;

typedef struct {
  char tty[8];
  int  session;
  
  char *header;
  char *status;
  
  char *name;
  char *city;
  char *sysop;
  char *flags;
  char *phone;
  char *addrs;
  
  int  speed;
  int  options;
  time_t start;
  
  pfile_t send;
  pfile_t recv;
} slot_t;

typedef struct _qslot_t {
	int n;
	char *addr;
	int mail, files, flags, tryes;
	struct _qslot_t *next, *prev;
	} qslot_t;

/* Generated by reswrap from file help.png */
const unsigned char nodelistico[]={
  0x47,0x49,0x46,0x38,0x39,0x61,0x16,0x00,0x16,0x00,0xf5,0x22,0x00,0x00,0x00,0x00,
  0x05,0x0f,0x16,0x09,0x12,0x1a,0x10,0x14,0x17,0x08,0x18,0x22,0x10,0x1b,0x23,0x0a,
  0x21,0x2f,0x0c,0x25,0x38,0x18,0x22,0x28,0x1a,0x28,0x33,0x28,0x2c,0x2f,0x24,0x2d,
  0x34,0x28,0x32,0x38,0x31,0x35,0x38,0x12,0x32,0x49,0x18,0x42,0x63,0x27,0x4e,0x69,
  0x25,0x53,0x74,0x26,0x5d,0x84,0x2c,0x67,0x8d,0x2f,0x6d,0x9c,0x37,0x66,0x86,0x31,
  0x6b,0x96,0x31,0x76,0xa8,0x47,0x73,0x96,0x37,0x87,0xba,0x37,0x8a,0xc7,0x42,0x86,
  0xbb,0x4f,0x93,0xc8,0x55,0x9a,0xca,0x70,0xa4,0xc5,0x72,0xad,0xd3,0x8e,0xba,0xd9,
  0x9c,0xc5,0xdf,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x21,0xf9,0x04,
  0x01,0x01,0x00,0x22,0x00,0x2c,0x00,0x00,0x00,0x00,0x16,0x00,0x16,0x00,0x00,0x06,
  0xfe,0x40,0x91,0x48,0x24,0x12,0x89,0x44,0x00,0xc5,0x22,0x91,0x28,0x14,0x04,0x22,
  0x91,0x48,0x24,0x12,0x89,0x44,0x22,0x91,0x48,0x04,0x18,0x80,0x30,0x8c,0x45,0x22,
  0x01,0x89,0x00,0x04,0x22,0x91,0x48,0x24,0x12,0x89,0x44,0x22,0x40,0xa8,0x93,0x08,
  0x88,0x44,0x22,0x11,0x00,0x53,0x71,0x00,0x80,0x22,0x91,0x48,0x24,0x12,0x89,0x44,
  0x8a,0x0e,0x04,0x20,0x12,0x89,0x44,0x22,0x11,0x00,0x73,0x01,0x88,0x44,0x22,0x91,
  0x48,0x24,0x02,0x80,0x2c,0x04,0x91,0x48,0x24,0x12,0x89,0x44,0xa2,0x85,0x66,0x72,
  0x10,0x89,0x44,0x22,0x91,0x08,0xd0,0xa9,0x04,0x44,0x22,0x91,0x48,0x24,0x12,0x01,
  0x45,0x0a,0x8e,0xe5,0x20,0x12,0x89,0x44,0x22,0x11,0x00,0x43,0x81,0x0c,0x06,0x22,
  0x91,0x48,0x24,0x12,0x01,0x3a,0x16,0x82,0x48,0x24,0x12,0x89,0x44,0x22,0x80,0xe4,
  0x92,0xd9,0x14,0x44,0x22,0x91,0xa8,0x01,0xba,0x00,0x44,0x22,0x91,0x48,0x24,0x12,
  0x89,0x44,0x80,0x88,0xe5,0x02,0x00,0x8a,0x44,0xa2,0x46,0x28,0x13,0x21,0x88,0x44,
  0x22,0x91,0x48,0x24,0x12,0x89,0x44,0x00,0x80,0x41,0x24,0x12,0x35,0x42,0x1a,0x88,
  0x40,0x24,0x12,0x89,0x44,0x22,0x91,0x48,0x24,0x12,0x89,0x44,0x22,0x91,0xa8,0x11,
  0xd2,0x44,0x08,0x22,0x91,0x48,0x24,0x12,0x89,0x44,0x22,0x91,0x48,0x04,0x14,0x89,
  0x44,0x22,0x05,0x48,0x13,0x11,0x88,0x44,0x22,0x91,0x48,0x24,0x12,0x89,0x44,0x22,
  0x91,0x48,0x24,0x12,0x01,0x40,0x9d,0x0b,0x40,0x24,0x12,0x89,0x44,0x22,0x91,0xb2,
  0x48,0x24,0x12,0x89,0x44,0x22,0x91,0x48,0x04,0xf0,0x68,0x24,0x01,0x91,0x48,0x24,
  0x12,0x89,0x44,0x22,0x91,0x08,0x28,0x12,0x89,0x44,0x22,0x91,0x08,0xa0,0x99,0x3c,
  0x00,0x22,0x91,0x48,0x24,0x12,0x89,0x44,0x22,0x91,0x48,0x24,0x12,0x89,0x44,0x22,
  0x11,0x00,0x40,0x10,0x89,0x44,0x22,0x91,0x48,0x24,0x12,0x89,0x44,0x22,0x91,0x48,
  0x24,0x12,0x89,0x44,0x22,0x91,0x48,0x24,0x12,0x89,0x44,0x22,0x11,0x50,0x24,0x12,
  0x89,0x44,0x22,0x91,0x48,0x24,0x12,0x89,0x44,0x22,0x91,0x48,0x24,0x12,0x89,0x44,
  0x22,0x91,0x48,0x24,0x12,0x89,0x44,0x22,0x91,0x48,0x24,0x4a,0x24,0x0a,0x22,0x91,
  0x48,0x24,0x12,0x89,0x44,0x22,0x91,0x48,0x24,0x12,0x89,0x44,0x22,0x51,0xe2,0xd3,
  0xd1,0x00,0x44,0x22,0xa0,0x48,0x24,0x12,0x89,0x44,0x22,0x91,0x48,0x24,0x12,0x89,
  0x44,0x22,0x82,0xa6,0xe2,0x00,0x88,0x44,0x22,0x91,0x48,0x24,0x12,0x89,0x44,0x22,
  0x91,0x48,0x24,0x12,0x89,0x44,0x02,0x41,0x40,0x24,0x12,0x89,0x44,0x22,0x91,0x48,
  0x14,0x04,0x00,0x3b
  };



/* Generated by reswrap from file locationbar_erase.gif */
const unsigned char skipico[]={
  0x47,0x49,0x46,0x38,0x39,0x61,0x16,0x00,0x16,0x00,0xf1,0x02,0x00,0x00,0x00,0x00,
  0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x21,0xf9,0x04,0x01,0x01,0x00,0x02,
  0x00,0x2c,0x00,0x00,0x00,0x00,0x16,0x00,0x16,0x00,0x00,0x02,0xfe,0x94,0x28,0x51,
  0xa2,0x44,0x89,0x12,0x25,0x4a,0x94,0x28,0x51,0xa2,0x44,0x89,0x12,0x25,0x4a,0x94,
  0x28,0x51,0xa2,0x44,0x89,0x12,0x25,0x4a,0x94,0x28,0x51,0xa2,0x44,0x89,0x12,0x25,
  0x4a,0x94,0x28,0x51,0xa2,0x44,0x89,0x12,0x25,0x4a,0x94,0x28,0x51,0xa2,0x44,0x81,
  0x00,0x01,0x02,0x04,0x08,0x10,0x20,0x44,0x89,0x12,0x25,0x0a,0x04,0x08,0x10,0x20,
  0x40,0x80,0x00,0x01,0x4a,0x94,0x28,0x51,0x20,0x40,0x80,0x00,0x01,0x02,0x04,0x08,
  0x10,0xa2,0x44,0x89,0x02,0x01,0x26,0x04,0x08,0x10,0x61,0x40,0x80,0x00,0x25,0x4a,
  0x14,0x08,0x30,0x61,0x40,0x80,0x09,0x03,0x02,0x04,0x08,0x51,0xa2,0x40,0x80,0x08,
  0x13,0x22,0x4c,0x08,0x10,0x20,0x40,0x80,0x12,0x05,0x02,0x04,0x98,0x30,0x61,0x40,
  0x80,0x00,0x01,0x02,0x84,0x28,0x10,0x20,0x40,0x84,0x09,0x01,0x02,0x04,0x08,0x10,
  0x20,0x40,0x81,0x00,0x01,0x26,0x4c,0x18,0x10,0x20,0x40,0x80,0x00,0x21,0x0a,0x04,
  0x88,0x30,0x21,0xc2,0x84,0x00,0x01,0x02,0x04,0x28,0x51,0x20,0xc0,0x84,0x01,0x01,
  0x26,0x0c,0x08,0x10,0x20,0x44,0x89,0x02,0x01,0x26,0x04,0x08,0x10,0x61,0x40,0x80,
  0x00,0x25,0x4a,0x14,0x08,0x10,0x20,0x40,0x80,0x00,0x01,0x02,0x84,0x28,0x51,0xa2,
  0x40,0x80,0x00,0x01,0x02,0x04,0x08,0x10,0xa0,0x44,0x89,0x12,0x05,0x02,0x04,0x08,
  0x10,0x20,0x40,0x80,0x10,0x25,0x4a,0x94,0x28,0x51,0xa2,0x44,0x89,0x12,0x25,0x4a,
  0x94,0x28,0x51,0xa2,0x44,0x89,0x12,0x25,0x4a,0x94,0x28,0x13,0x51,0xa2,0x44,0x89,
  0x12,0x25,0x4a,0x94,0x28,0x51,0xa2,0x44,0x89,0x12,0x25,0x4a,0x94,0x28,0x15,0x00,
  0x3b
  };


/* Generated by reswrap from file goto.gif */
const unsigned char exitico[]={
  0x47,0x49,0x46,0x38,0x39,0x61,0x16,0x00,0x16,0x00,0xf3,0x0e,0x00,0x00,0x00,0x00,
  0x10,0x0d,0x0b,0x14,0x10,0x0d,0x1b,0x17,0x12,0x24,0x1d,0x17,0x2a,0x22,0x1c,0x35,
  0x2b,0x23,0x3c,0x31,0x28,0x43,0x36,0x2c,0x4b,0x3d,0x32,0x4e,0x40,0x34,0x56,0x46,
  0x39,0x62,0x51,0x42,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x21,0xf9,0x04,
  0x01,0x01,0x00,0x0e,0x00,0x2c,0x00,0x00,0x00,0x00,0x16,0x00,0x16,0x00,0x00,0x04,
  0xfe,0xd0,0x39,0xe7,0x9c,0x73,0xce,0x39,0xe7,0x9c,0x83,0x42,0x04,0xe7,0x9c,0x73,
  0xce,0x39,0xe7,0x1c,0x74,0xce,0x39,0x07,0xc8,0x59,0xab,0x04,0xe7,0xa0,0x73,0xce,
  0x39,0xe7,0x9c,0x73,0xce,0x85,0x03,0xd3,0x5a,0x49,0x05,0xe7,0x9c,0x73,0xce,0x39,
  0xe8,0x9c,0x73,0x4e,0xa0,0xb4,0xd6,0x5a,0xa5,0x41,0xe7,0x9c,0x73,0xce,0x39,0xe7,
  0x9c,0x73,0x04,0xae,0xc5,0x56,0x4a,0xc0,0x39,0xe7,0x9c,0x73,0xd0,0x39,0xe7,0x9c,
  0x03,0x28,0x2d,0xb6,0x12,0x81,0xcd,0x39,0xe7,0x9c,0x73,0xce,0x39,0xe7,0x1c,0x0c,
  0x09,0xa5,0x54,0x44,0x73,0xce,0x39,0xe7,0xa0,0x73,0xce,0x39,0xe7,0x5c,0x13,0xc5,
  0x8c,0x06,0x9b,0x73,0xce,0x39,0xe7,0x9c,0x73,0xce,0x39,0x58,0x88,0x6b,0xad,0x35,
  0xe7,0x9c,0x03,0xc0,0x41,0xe7,0x9c,0x73,0xce,0x8d,0xa4,0x88,0x73,0x0e,0x3a,0xe7,
  0x00,0x40,0x06,0x38,0xe7,0x9c,0x73,0xd0,0x91,0xb4,0x12,0x70,0xce,0x39,0x80,0x4e,
  0x82,0x07,0x38,0xe7,0x9c,0x73,0xae,0x15,0x30,0x1a,0x74,0xce,0x81,0x94,0x4e,0x42,
  0xc0,0x39,0xe7,0xa0,0x73,0xce,0xb5,0xd6,0x9c,0x73,0x0e,0x20,0x05,0xd5,0x1a,0xcd,
  0x39,0xe7,0x9c,0x73,0xce,0x39,0xe8,0x9c,0x73,0x24,0x21,0x94,0x4c,0x70,0xce,0x41,
  0xe7,0x9c,0x73,0xce,0x39,0xe7,0x9c,0x03,0x09,0x1e,0x94,0x0a,0x70,0xce,0x39,0xe7,
  0x9c,0x73,0xd0,0x39,0xe7,0x9c,0x03,0xe8,0x9c,0x02,0x9a,0x83,0xce,0x39,0xe7,0x9c,
  0x73,0xce,0x39,0xe7,0x1c,0x1c,0x07,0x01,0xd0,0x9c,0x73,0xce,0x39,0xe7,0xa0,0x47,
  0x73,0xce,0x39,0x17,0x5c,0x03,0xa0,0x35,0x07,0x9d,0x73,0xce,0x39,0xe7,0x9c,0x73,
  0xce,0x11,0x98,0x84,0x6b,0xcd,0x39,0xe7,0x9c,0x73,0xce,0x41,0xe7,0x9c,0x73,0xce,
  0x81,0x75,0x84,0x73,0x0e,0x3a,0xe7,0x9c,0x73,0xce,0x39,0xe7,0x9c,0x73,0xd0,0x15,
  0x43,0x82,0x73,0xce,0x39,0xe7,0x9c,0x83,0xce,0x39,0xe7,0x9c,0x73,0xae,0x01,0xd0,
  0x1c,0x74,0xce,0x39,0xe7,0x9c,0x8b,0x00,0x3b
  };



/* Generated by reswrap from file reload.gif */
const unsigned char rereadico[]={
  0x47,0x49,0x46,0x38,0x39,0x61,0x16,0x00,0x16,0x00,0xf6,0x4c,0x00,0x01,0x01,0x01,
  0x05,0x16,0x02,0x13,0x14,0x00,0x18,0x18,0x19,0x07,0x24,0x06,0x0b,0x31,0x08,0x14,
  0x29,0x09,0x19,0x36,0x0d,0x16,0x3e,0x10,0x25,0x08,0x06,0x21,0x21,0x06,0x21,0x3b,
  0x14,0x28,0x28,0x28,0x38,0x38,0x38,0x3f,0x3f,0x47,0x15,0x41,0x0c,0x1d,0x46,0x13,
  0x28,0x49,0x19,0x21,0x50,0x17,0x2c,0x57,0x2a,0x30,0x5b,0x26,0x23,0x76,0x1a,0x26,
  0x64,0x24,0x2a,0x73,0x2f,0x2c,0x7f,0x39,0x36,0x6d,0x2f,0x33,0x7f,0x3a,0x49,0x0f,
  0x0e,0x44,0x6a,0x3d,0x40,0x72,0x37,0x45,0x45,0x46,0x4a,0x4a,0x52,0x53,0x53,0x54,
  0x40,0x65,0x41,0x47,0x7d,0x40,0x6a,0x6a,0x6a,0x73,0x73,0x73,0x2c,0x81,0x3a,0x35,
  0x87,0x3b,0x2f,0x8d,0x43,0x3a,0x8b,0x47,0x3a,0x92,0x4c,0x31,0x9e,0x52,0x2f,0xa0,
  0x57,0x32,0xa7,0x5c,0x33,0xb3,0x64,0x48,0xa9,0x2e,0x49,0xa2,0x3f,0x53,0xab,0x3d,
  0x46,0x91,0x4f,0x41,0x96,0x52,0x50,0x85,0x4a,0x46,0xb6,0x70,0x55,0xbd,0x7a,0x66,
  0x8a,0x5f,0x62,0x86,0x63,0x66,0xc2,0x83,0x7d,0xcb,0x8f,0xc7,0x37,0x1a,0x86,0x86,
  0x86,0x8f,0x8f,0x97,0x9b,0x9b,0x9b,0xa7,0xa7,0xa7,0xbc,0xbc,0xbc,0x8c,0xcb,0x84,
  0x82,0xcd,0x91,0x9d,0xc3,0x92,0xa3,0xda,0xae,0xb3,0xd7,0xac,0xb4,0xe1,0xba,0xbe,
  0xe5,0xc1,0xca,0xca,0xca,0xcc,0xea,0xcc,0xd9,0xf2,0xd8,0xe0,0xf3,0xdd,0xe3,0xf4,
  0xe1,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x21,0xf9,0x04,
  0x01,0x01,0x00,0x4c,0x00,0x2c,0x00,0x00,0x00,0x00,0x16,0x00,0x16,0x00,0x00,0x07,
  0xfe,0x80,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x10,0x12,0x08,0x10,0x0b,0x06,
  0x02,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x10,0x17,
  0x38,0x49,0x45,0x41,0x31,0x22,0x1d,0x12,0x0f,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,
  0x4c,0x4c,0x4c,0x10,0x2b,0x49,0x46,0x34,0x27,0x08,0x06,0x02,0x0a,0x02,0x10,0x06,
  0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x17,0x4b,0x45,0x34,0x33,0x02,0x02,
  0x4c,0x4c,0x4c,0x4c,0x02,0x06,0x02,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x10,0x38,
  0x48,0x38,0x25,0x06,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x06,0x02,0x4c,0x4c,
  0x4c,0x4c,0x4c,0x4c,0x17,0x4b,0x43,0x1a,0x02,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,
  0x80,0x4c,0x4c,0x4c,0x02,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x16,0x4a,0x38,0x31,0x06,
  0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x02,0x08,0x11,
  0x13,0x2b,0x46,0x35,0x27,0x06,0x02,0x02,0x02,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,
  0x4c,0x4c,0x4c,0x02,0x18,0x45,0x49,0x49,0x38,0x29,0x26,0x41,0x48,0x18,0x07,0x4c,
  0x4c,0x4c,0x00,0x03,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x02,0x19,0x2c,0x2d,0x2b,0x29,
  0x38,0x18,0x16,0x02,0x4c,0x4c,0x4c,0x00,0x3d,0x1e,0x03,0x4c,0x4c,0x4c,0x4c,0x4c,
  0x4c,0x02,0x19,0x27,0x2d,0x38,0x25,0x14,0x02,0x4c,0x4c,0x4c,0x00,0x3f,0x23,0x3b,
  0x1e,0x02,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x02,0x1d,0x27,0x25,0x16,0x02,0x4c,0xeb,
  0x80,0x4c,0x4c,0x00,0x47,0x24,0x0d,0x1e,0x3c,0x1f,0x03,0x4c,0x4c,0x4c,0x4c,0x4c,
  0x4c,0x02,0x19,0x14,0x02,0x4c,0x4c,0x4c,0x00,0x3f,0x23,0x0d,0x03,0x0c,0x1f,0x3c,
  0x1e,0x03,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x02,0x02,0x4c,0x4c,0x4c,0x0c,0x3d,0x23,
  0x1e,0x1e,0x0c,0x0c,0x0c,0x1e,0x24,0x0c,0x02,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,
  0x4c,0x4c,0x4c,0x00,0x00,0x00,0x00,0x24,0x0e,0x20,0x0c,0x00,0x00,0x00,0x00,0x4c,
  0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x00,0x3b,0x1e,0x20,
  0x0c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x00,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,
  0x4c,0x4c,0x00,0x3e,0x0d,0x20,0x03,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x00,0x00,0x80,
  0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x00,0x3e,0x23,0x1e,0x1e,0x00,0x4c,0x4c,
  0x4c,0x4c,0x4c,0x4c,0x4c,0x00,0x00,0x00,0x4c,0x4c,0x4c,0x4c,0x00,0x00,0x3e,0x3b,
  0x1e,0x1e,0x03,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x00,0x0c,0x02,0x03,
  0x00,0x00,0x24,0x3f,0x3b,0x20,0x1f,0x0c,0x00,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,
  0x4c,0x4c,0x4c,0x00,0x03,0x20,0x23,0x3b,0x3d,0x20,0x1e,0x1f,0x02,0x00,0x4c,0x4c,
  0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x00,0x03,0x03,0x03,0x0c,
  0x03,0x00,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x81,0x00,0x3b
  };



/* Generated by reswrap from file queue.gif */
const unsigned char rescanico[]={
  0x47,0x49,0x46,0x38,0x39,0x61,0x16,0x00,0x16,0x00,0xf3,0x00,0x00,0x00,0x00,0x00,
  0x37,0x37,0x37,0x4c,0x4c,0x4c,0x68,0x68,0x68,0x79,0x79,0x79,0x67,0x67,0x89,0xa0,
  0xa0,0xa0,0xb6,0xb6,0xb6,0xc7,0xc7,0xc7,0xd7,0xd7,0xd7,0xc0,0xc0,0xff,0xe7,0xe7,
  0xe7,0xf9,0xf9,0xf9,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x21,0xf9,0x04,
  0x01,0x01,0x00,0x0d,0x00,0x2c,0x00,0x00,0x00,0x00,0x16,0x00,0x16,0x00,0x00,0x04,
  0xfe,0xb0,0xb5,0xd6,0x5a,0x6b,0xad,0xb5,0xd6,0x5a,0x83,0xad,0xb5,0xd6,0x5a,0x6b,
  0xad,0xb5,0xd6,0x1a,0x6c,0xad,0xb5,0xd6,0x5a,0x6b,0xad,0xb5,0xd6,0x60,0x6b,0xad,
  0xb5,0xd6,0x5a,0x6b,0xad,0x35,0x01,0x45,0x6b,0xad,0xb5,0xd6,0x5a,0x6b,0xad,0x35,
  0xd8,0x5a,0x6b,0xad,0x09,0x63,0x84,0x68,0xad,0xc1,0xd6,0x5a,0x6b,0xad,0xb5,0xd6,
  0x5a,0x2b,0x05,0x0a,0x63,0x8c,0x11,0xa2,0xb5,0xd6,0x5a,0x6b,0xb0,0xb5,0xd6,0x5a,
  0x51,0xaa,0x14,0x63,0x8c,0x81,0x46,0x88,0xd6,0x5a,0x6b,0xad,0x35,0x32,0x0a,0x54,
  0x4a,0xa9,0x52,0x8c,0x31,0xc6,0x08,0xd1,0x60,0x6b,0xad,0x0d,0xc6,0xc6,0x50,0x4a,
  0x29,0x05,0x4b,0x31,0xc6,0x18,0x03,0x5a,0x6b,0x8d,0x2c,0xb8,0xd6,0x1a,0x43,0x29,
  0xa5,0x54,0x29,0xc6,0x40,0x03,0x5a,0x6b,0x64,0xad,0x95,0xd2,0x5a,0x06,0x12,0xa5,
  0x94,0x52,0xc0,0x80,0xd6,0x5a,0x1b,0x90,0xa5,0x94,0xd2,0x62,0x8c,0x8d,0xa1,0x94,
  0x82,0xc0,0x80,0xd6,0xda,0x58,0x29,0xad,0xc5,0x18,0x64,0x8c,0xb1,0x15,0x14,0x30,
  0xa0,0xb5,0xd6,0xe0,0x58,0x6b,0x2d,0xc6,0x18,0x5b,0x2b,0x9d,0x00,0x15,0x30,0xa0,
  0xb5,0x36,0x56,0x62,0x8c,0x31,0xc8,0xd6,0x4a,0x09,0x05,0x05,0x0c,0x68,0xad,0xc1,
  0x46,0x18,0x63,0x8c,0xb1,0x95,0x52,0x5a,0x09,0x06,0x05,0x0c,0x68,0xad,0x11,0xc6,
  0x18,0x5b,0x70,0xa5,0x94,0x16,0x5b,0x41,0x01,0x03,0x5a,0x83,0xad,0x91,0x95,0xd6,
  0x5a,0x09,0xad,0xc5,0x18,0x4c,0x40,0x01,0x03,0x5a,0x6b,0xad,0x09,0x81,0x20,0x47,
  0x4a,0x6b,0x31,0xb6,0x52,0x50,0x00,0x80,0x06,0x5b,0x6b,0xad,0xb5,0x16,0x42,0x4a,
  0x6c,0x2d,0x88,0x4e,0x50,0xa0,0xb5,0xd6,0x5a,0x6b,0xad,0xc1,0xd6,0x5a,0x08,0xe9,
  0xa0,0x13,0x00,0x68,0x0d,0xb6,0xd6,0x5a,0x6b,0xad,0xb5,0xd6,0x5a,0x0b,0x30,0x98,
  0x13,0x5a,0x6b,0xad,0xb5,0xd6,0x5a,0x83,0xad,0xb5,0xd6,0x5a,0x6b,0x2d,0x84,0xd6,
  0x1a,0x6c,0xad,0xb5,0xd6,0x5a,0x8b,0x00,0x3b
  };

/* Generated by reswrap from file connect_no.gif */
const unsigned char connectingico[]={
  0x47,0x49,0x46,0x38,0x39,0x61,0x20,0x00,0x20,0x00,0xf5,0x21,0x00,0x01,0x01,0x01,
  0x11,0x11,0x00,0x14,0x14,0x14,0x2a,0x2a,0x00,0x29,0x29,0x29,0x2a,0x2c,0x35,0x37,
  0x37,0x38,0x45,0x45,0x00,0x5c,0x5c,0x00,0x65,0x65,0x00,0x48,0x47,0x48,0x59,0x58,
  0x59,0x68,0x66,0x68,0x75,0x73,0x75,0x80,0x7e,0x80,0x9a,0x9b,0x00,0xab,0xab,0x00,
  0xd2,0xd2,0x00,0xe7,0xe7,0x00,0xfd,0xfd,0x00,0x85,0x83,0x85,0x9b,0x98,0x9b,0xa3,
  0x9f,0xaa,0xa6,0xa4,0xa6,0xb1,0xaf,0xb1,0xbe,0xbc,0xbe,0xbd,0xbb,0xc0,0xc9,0xc6,
  0xc9,0xd0,0xcf,0xd0,0xdd,0xdd,0xdd,0xe0,0xdd,0xe0,0xe8,0xe6,0xe8,0xfe,0xfe,0xfe,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x21,0xf9,0x04,
  0x01,0x01,0x00,0x21,0x00,0x2c,0x00,0x00,0x00,0x00,0x20,0x00,0x20,0x00,0x00,0x06,
  0xfe,0xc0,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,
  0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,
  0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,
  0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x80,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,
  0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,
  0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,
  0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x01,
  0x43,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,
  0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,
  0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,
  0x85,0x42,0xa1,0x50,0x28,0x14,0x02,0x86,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,
  0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,
  0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,
  0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x04,0x0c,0x85,
  0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,
  0x50,0x28,0x14,0x0a,0x85,0x26,0x91,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0xfe,
  0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,
  0x0a,0x4d,0x22,0x90,0x50,0x08,0x18,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,
  0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x4d,0x24,0x90,
  0x43,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,
  0x14,0x0a,0x85,0x42,0x21,0x00,0x20,0x14,0x9a,0x44,0x20,0x87,0x10,0x30,0x14,0x0a,
  0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,
  0x21,0x00,0x68,0x03,0x98,0x44,0x20,0x87,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,
  0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0x00,0x50,0xe7,0xa3,
  0x01,0x40,0x0e,0x21,0x60,0x28,0x14,0x0a,0x85,0x26,0x91,0x50,0x28,0x14,0x0a,0x85,
  0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0x00,0x50,0xa7,0xd3,0xe9,0x68,0x00,0xa1,
  0x50,0x28,0x14,0x0a,0x4d,0x22,0x90,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,
  0x14,0x0a,0x85,0x00,0x20,0x8f,0xa7,0xd3,0xe9,0x7c,0x32,0x40,0x40,0x28,0x14,0x0a,
  0x4d,0x24,0x90,0x43,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x00,
  0xa0,0x4e,0xa7,0xd3,0xe9,0x74,0x3a,0x9d,0x0c,0x20,0x14,0x9a,0x44,0x20,0x87,0x50,
  0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x00,0x94,0xcc,0xa7,0xd3,
  0xe9,0x74,0x80,0x9f,0xce,0x27,0x13,0x98,0x48,0x20,0x87,0x50,0x28,0x14,0xfe,0x0a,
  0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0x80,0x85,0x23,0xd3,0xf1,0x74,0x3a,
  0x9e,0xcf,0xc7,0x93,0x01,0x40,0x06,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,
  0x28,0x14,0x0a,0x85,0x42,0x80,0xc5,0xc2,0x91,0xe9,0x00,0x3d,0x9d,0x4e,0xc7,0xd3,
  0xe9,0x6c,0x00,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,
  0x42,0x21,0xc0,0x62,0xb1,0x70,0x68,0x3c,0x1e,0x8f,0xa7,0xd3,0xe9,0x74,0x30,0x80,
  0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x10,0x60,
  0xc1,0x00,0x2e,0x28,0x9a,0x8e,0xa7,0xf3,0xe9,0x78,0x30,0x14,0x40,0x28,0x14,0x0a,
  0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x04,0x70,0x74,0x18,
  0x8e,0x4c,0xe7,0xd3,0xe9,0x5c,0x1c,0x14,0x40,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,
  0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x04,0x04,0x68,0x40,0x9c,0x85,0x23,0xd3,
  0xe9,0x58,0x1c,0x8e,0x05,0x20,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,
  0x42,0xa1,0x50,0x28,0x04,0xc8,0x80,0x36,0x17,0xc6,0xc2,0x91,0xa9,0x50,0x1c,0x0b,
  0x40,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x08,
  0x18,0xc8,0x80,0x3c,0x0c,0x05,0x63,0xb1,0x58,0x50,0x1c,0x0b,0x40,0x28,0x14,0x0a,
  0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x08,0xa0,0x01,0x71,0x18,
  0x00,0x00,0x60,0xb1,0x58,0x50,0x16,0x80,0x50,0x28,0x14,0x0a,0x85,0x96,0x42,0xa1,
  0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x10,0x10,0xa0,0x01,0x6d,0x18,0x82,0x50,0x28,
  0x24,0x58,0x2c,0x16,0x82,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,
  0x85,0x42,0xa1,0x10,0x40,0x03,0xea,0x30,0x04,0xa1,0x50,0x28,0x14,0x02,0x00,0x00,
  0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0x21,0x20,
  0x20,0x03,0xea,0x30,0x00,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,
  0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0x21,0x40,0x06,0xb4,0x61,0x00,
  0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,
  0x50,0x28,0x14,0x0a,0x85,0x42,0xc0,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,
  0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x0a,
  0x85,0x42,0xa1,0x20,0x00,0x3b
  };



/* Generated by reswrap from file mail_get.gif */
const unsigned char pollico[]={
  0x47,0x49,0x46,0x38,0x39,0x61,0x16,0x00,0x16,0x00,0xf5,0x32,0x00,0x00,0x00,0x00,
  0x03,0x0b,0x17,0x05,0x10,0x1e,0x1d,0x1d,0x1d,0x09,0x1d,0x3c,0x27,0x27,0x27,0x38,
  0x38,0x38,0x0b,0x24,0x4a,0x0b,0x2a,0x50,0x0f,0x36,0x5e,0x14,0x35,0x5a,0x12,0x3a,
  0x6b,0x23,0x3c,0x62,0x0e,0x44,0x71,0x5b,0x5a,0x5b,0x62,0x61,0x62,0x78,0x78,0x78,
  0x0f,0x53,0x8c,0x10,0x4f,0x85,0x0d,0x77,0xba,0x17,0x7a,0xb4,0x32,0x6d,0x9b,0x27,
  0x7a,0xac,0x1c,0x84,0xbf,0x0e,0x86,0xc4,0x14,0x89,0xc5,0x1b,0x9e,0xce,0x14,0xa2,
  0xd7,0x3e,0xb3,0xd9,0x62,0x94,0xbb,0x57,0xa7,0xd5,0x46,0xbf,0xe7,0x45,0xc0,0xe4,
  0x85,0x84,0x85,0x97,0x97,0x97,0xa2,0x9f,0xa2,0xa6,0xa6,0xa6,0xb7,0xb6,0xb7,0x8d,
  0xbe,0xe0,0x9e,0xc0,0xdd,0xa5,0xce,0xea,0xbd,0xd5,0xec,0xc9,0xc8,0xc9,0xd0,0xcf,
  0xd0,0xd9,0xd8,0xd9,0xd1,0xdf,0xee,0xdd,0xea,0xf7,0xe3,0xe3,0xe3,0xed,0xef,0xfa,
  0xf8,0xf8,0xf8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x21,0xf9,0x04,
  0x01,0x01,0x00,0x32,0x00,0x2c,0x00,0x00,0x00,0x00,0x16,0x00,0x16,0x00,0x00,0x06,
  0xfe,0x40,0x99,0x4c,0x26,0x93,0xc9,0x64,0xb2,0xc5,0x62,0xa1,0x50,0x1c,0x64,0x32,
  0x99,0x4c,0x26,0x93,0xc9,0x64,0x32,0x99,0x4c,0x26,0x93,0xc9,0x16,0x27,0x98,0xab,
  0x23,0x90,0xc9,0x64,0x32,0x99,0x4c,0x26,0x93,0xc9,0x64,0x32,0x99,0x4c,0x26,0x5b,
  0xb8,0x3c,0x26,0x8b,0x40,0x26,0x93,0xc9,0x80,0x32,0x99,0x4c,0x26,0x93,0xc9,0x64,
  0x32,0x99,0x4c,0xb6,0x80,0x65,0x38,0x16,0x81,0x4c,0x26,0x93,0xc9,0x64,0x32,0x99,
  0x4c,0x26,0x93,0xc9,0x64,0x32,0x99,0xc2,0x35,0x01,0x51,0x04,0x32,0x99,0x4c,0x26,
  0x93,0xc9,0x64,0x32,0x99,0x4c,0x86,0x58,0x2c,0x18,0x8c,0xd4,0x44,0x43,0x69,0x00,
  0x13,0x0a,0x02,0x40,0x26,0x93,0xc9,0x64,0x32,0x99,0xac,0x80,0xa8,0xb8,0x50,0x9e,
  0xc9,0x64,0xf3,0xb9,0x48,0x00,0x00,0x99,0x4c,0x26,0x93,0xc9,0x64,0x32,0xc0,0x0a,
  0x51,0x89,0x4d,0x26,0x97,0xcf,0x45,0x12,0x78,0x01,0x64,0x32,0x99,0x4c,0x26,0x93,
  0x05,0x54,0xab,0xd5,0xa1,0x12,0x03,0x62,0x3e,0x14,0x49,0x80,0xf5,0x7a,0x01,0x64,
  0x32,0x99,0x4c,0x26,0x03,0x84,0x44,0x2b,0x96,0xa2,0x02,0xcb,0x44,0x02,0x2c,0xd6,
  0xab,0x04,0x90,0xc9,0x64,0x32,0xd9,0xe0,0x11,0x22,0xb1,0x58,0x2c,0x45,0x44,0x12,
  0x60,0xb1,0x58,0xaf,0x52,0x09,0x20,0x93,0xc9,0x64,0x80,0x87,0x68,0x05,0x64,0xb1,
  0x58,0x2c,0x04,0xe0,0xc5,0x62,0xb1,0x5e,0xaf,0x12,0x40,0x26,0x93,0x01,0x1e,0x0f,
  0x95,0x6a,0xb4,0x62,0xb1,0x56,0x2f,0xd6,0x6a,0xb4,0x5a,0xbd,0x4a,0x25,0x80,0xb2,
  0x4c,0x06,0x78,0x3c,0x42,0xa1,0x10,0x8b,0xa5,0x12,0xa9,0x58,0xac,0x50,0x48,0x35,
  0x2a,0x95,0x00,0x32,0x40,0x01,0x68,0x30,0x18,0x20,0x21,0x15,0x2b,0x14,0x12,0xad,
  0x58,0xa1,0x90,0xc3,0x60,0x30,0x18,0x00,0x80,0x58,0x2c,0x06,0x33,0x84,0x56,0xa1,
  0x50,0x28,0x24,0x5a,0x85,0x42,0x86,0x58,0x0c,0x16,0x03,0x00,0x62,0xb1,0x58,0xcc,
  0x10,0x0a,0x41,0x42,0xa1,0x50,0x28,0x14,0x0a,0x19,0x54,0xab,0x15,0x50,0x05,0x00,
  0xbc,0x5e,0x2f,0x16,0xc4,0x60,0x30,0x18,0x0c,0x06,0x83,0xc1,0x60,0x80,0x94,0x4a,
  0xa5,0x12,0x00,0xc0,0x5a,0xad,0x54,0x2a,0x95,0x4a,0xa5,0x52,0xa9,0x54,0x2a,0x95,
  0xaa,0x54,0x22,0x91,0x48,0x00,0x80,0x4a,0x55,0x2a,0x95,0x4a,0xa5,0x52,0x89,0x54,
  0x2a,0x91,0x4a,0x24,0x20,0x49,0x24,0x12,0x89,0x00,0x00,0xd2,0x88,0x44,0x1a,0x89,
  0x44,0x22,0x91,0x48,0x24,0x12,0x89,0x44,0xa1,0x10,0x24,0x14,0x02,0xc8,0x0a,0x85,
  0x42,0xa1,0x50,0x28,0x14,0x0a,0x85,0x42,0xa1,0x50,0x28,0x14,0x06,0x85,0x42,0x41,
  0x16,0x04,0x00,0x3b
  };

/* Generated by reswrap from file down.gif */
const unsigned char freqico[]={
  0x47,0x49,0x46,0x38,0x39,0x61,0x16,0x00,0x16,0x00,0xf5,0x2a,0x00,0x01,0x00,0x01,
  0x0d,0x39,0x56,0x1c,0x42,0x5c,0x25,0x4a,0x63,0x30,0x53,0x6b,0x11,0x65,0x8a,0x11,
  0x6c,0x92,0x19,0x74,0x98,0x33,0x7f,0x9e,0x0e,0x89,0xab,0x0e,0x8f,0xb6,0x0e,0x94,
  0xba,0x0e,0x9e,0xc3,0x0e,0xad,0xcb,0x0e,0xb0,0xcd,0x1c,0xa2,0xc3,0x17,0xab,0xd0,
  0x24,0xa9,0xcb,0x3d,0xb9,0xd3,0x15,0xc1,0xda,0x23,0xc2,0xd5,0x6b,0xbe,0xd8,0x7f,
  0xbd,0xd5,0x44,0xc0,0xd7,0x40,0xd1,0xdb,0x5a,0xc2,0xd9,0x47,0xde,0xe8,0x7b,0xc7,
  0xde,0x80,0xb6,0xd0,0x8f,0xca,0xda,0x96,0xc3,0xd7,0x8f,0xcc,0xe2,0xa9,0xcd,0xde,
  0xa7,0xd7,0xe7,0xb8,0xde,0xeb,0xbc,0xe2,0xec,0xc0,0xdf,0xee,0xcd,0xe1,0xec,0xce,
  0xe9,0xf2,0xdb,0xec,0xf4,0xe1,0xeb,0xf2,0xfd,0xfb,0xfd,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x21,0xf9,0x04,
  0x01,0x01,0x00,0x2a,0x00,0x2c,0x00,0x00,0x00,0x00,0x16,0x00,0x16,0x00,0x00,0x06,
  0xfe,0x40,0x95,0x4a,0xa5,0x52,0xa9,0x54,0x2a,0x95,0x4a,0xa5,0x52,0xa9,0x54,0x2a,
  0x95,0x4a,0xa5,0x52,0xa9,0x54,0x2a,0x95,0x4a,0xa5,0x52,0xa9,0x54,0x2a,0x95,0x4a,
  0xa5,0x52,0xa9,0x54,0x2a,0x95,0x4a,0xa5,0x52,0xa9,0x54,0x2a,0x95,0x4a,0xa5,0x52,
  0xa9,0x54,0x2a,0x95,0x4a,0xa5,0x52,0xa9,0x80,0x2a,0x95,0x4a,0xa5,0x52,0xa9,0x54,
  0x2a,0x95,0x2a,0x10,0x18,0x10,0x04,0x81,0x40,0x40,0xa5,0x52,0xa9,0x54,0x2a,0x95,
  0x4a,0xa5,0x52,0xa9,0x54,0xaa,0x00,0x87,0x84,0x3a,0x95,0x0e,0x00,0x95,0x4a,0xa5,
  0x52,0xa9,0x54,0x2a,0x95,0x4a,0xa5,0x52,0xa9,0x02,0xa8,0x4e,0xa7,0xd3,0x29,0x00,
  0x01,0x2a,0x95,0x4a,0xa5,0x52,0xa9,0x54,0x2a,0x95,0x4a,0xa5,0x52,0x05,0x52,0xa3,
  0xcb,0x42,0x62,0x00,0xa8,0x54,0x2a,0x95,0x4a,0xa5,0x52,0xa9,0x54,0x2a,0x95,0x4a,
  0x15,0x48,0x99,0x32,0x09,0x8a,0x01,0xa0,0x52,0xa9,0x54,0x2a,0x95,0x4a,0xa5,0x52,
  0xa9,0x54,0x2a,0x55,0x20,0x35,0x02,0x5e,0x12,0x1a,0x03,0x40,0xa5,0x52,0xa9,0x54,
  0x2a,0x95,0x4a,0xa5,0x52,0xa9,0x54,0x2a,0x41,0x6a,0xd3,0x58,0x48,0x0c,0x00,0x95,
  0x4a,0xa5,0x52,0xa9,0x54,0x2a,0x95,0x20,0x10,0x08,0x04,0x02,0xa2,0x4a,0x84,0x31,
  0x51,0x08,0x02,0x81,0x40,0x40,0xa0,0x52,0xa9,0x54,0x2a,0x40,0x82,0x03,0x04,0x79,
  0x42,0x8e,0x85,0x62,0xb2,0x70,0x34,0x10,0x06,0x80,0x4a,0xa5,0x52,0xa9,0x54,0x2a,
  0x00,0x02,0x05,0xca,0x30,0x14,0x0b,0xc8,0x44,0xf3,0x28,0x00,0x54,0x2a,0x95,0xb2,
  0x4a,0xa5,0x52,0xa9,0x54,0x00,0x44,0xea,0x94,0x61,0x30,0x26,0x9a,0x47,0x01,0xa0,
  0x52,0xa9,0x54,0x2a,0x95,0x0a,0xa8,0x52,0xa9,0x54,0x00,0x44,0xea,0x94,0xc9,0x68,
  0x1e,0x05,0x80,0x4a,0xa5,0x52,0xa9,0x54,0x2a,0x95,0x4a,0xa5,0x52,0xa9,0x00,0x89,
  0x94,0x48,0xf3,0x28,0x00,0x54,0x2a,0x95,0x4a,0xa5,0x52,0xa9,0x54,0x2a,0x95,0x4a,
  0xa5,0x52,0x01,0x10,0xa8,0x08,0x01,0xa0,0x52,0xa9,0x54,0x2a,0x15,0x50,0xa5,0x52,
  0xa9,0x54,0x2a,0x95,0x4a,0xa5,0x52,0x01,0x12,0x05,0x80,0x4a,0xa5,0x52,0xa9,0x54,
  0x2a,0x95,0x4a,0xa5,0x52,0xa9,0x54,0x2a,0x95,0x4a,0xa5,0x02,0x00,0x54,0x2a,0x95,
  0x4a,0xa5,0x52,0xa9,0x54,0x2a,0x95,0x4a,0xa5,0x52,0xa9,0x54,0x2a,0x95,0x4a,0xa5,
  0x52,0xa9,0x54,0x2a,0xa0,0x4a,0xa5,0x52,0xa9,0x54,0x2a,0x95,0x4a,0xa5,0x52,0xa9,
  0x54,0x2a,0x95,0x4a,0xa5,0x52,0xa9,0x54,0x2a,0x95,0x4a,0xa5,0x52,0xa9,0x54,0x2a,
  0x95,0x4a,0xa5,0x52,0xa9,0x54,0x2a,0x95,0x4a,0xa5,0x52,0xa9,0x54,0x2a,0x95,0x4a,
  0x15,0x04,0x00,0x3b
  };

const unsigned char minifolderopen[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x10,0x00,0x10,0x00,0xf2,0x00,0x00,0xb2,0xc0,0xdc,
  0x00,0x00,0x00,0x7f,0x7f,0x7f,0xff,0xff,0xff,0xd9,0xd9,0xd9,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x2c,0x00,0x00,0x00,0x00,0x10,0x00,0x10,0x00,0x00,0x03,
  0x42,0x08,0xba,0xdc,0x2c,0x10,0xba,0x37,0x6a,0x15,0x13,0x88,0x41,0x4a,0x27,0x43,
  0x14,0x29,0x9b,0x67,0x82,0x56,0x18,0x68,0xdc,0xe9,0x12,0x42,0x20,0xce,0x62,0x11,
  0x6f,0x69,0x1e,0xc3,0x72,0xfb,0xb9,0xb2,0x18,0xeb,0x47,0xbc,0xad,0x4a,0xc4,0x93,
  0x6c,0xc5,0x7a,0x99,0x62,0x4c,0x1a,0x2d,0xc0,0x04,0x50,0xaf,0x58,0x6c,0x66,0xcb,
  0x6d,0x24,0x00,0x00,0x3b
  };

/* Generated by reswrap from file up.gif */
const unsigned char sendico[]={
  0x47,0x49,0x46,0x38,0x39,0x61,0x16,0x00,0x16,0x00,0xf5,0x37,0x00,0x00,0x00,0x00,
  0x04,0x0c,0x17,0x05,0x10,0x1d,0x05,0x14,0x23,0x0c,0x39,0x56,0x0f,0x43,0x64,0x10,
  0x57,0x7a,0x26,0x49,0x65,0x2c,0x53,0x71,0x35,0x56,0x6e,0x0f,0x5f,0x84,0x10,0x64,
  0x88,0x13,0x68,0x90,0x10,0x70,0x94,0x10,0x7e,0xa3,0x29,0x64,0x87,0x48,0x67,0x89,
  0x0e,0x97,0xbf,0x11,0x8e,0xb0,0x11,0x93,0xb5,0x0f,0x9e,0xc2,0x0e,0xa0,0xc5,0x16,
  0xa4,0xc5,0x17,0xab,0xd0,0x1a,0xb2,0xcd,0x14,0xb9,0xd4,0x3f,0x9d,0xc0,0x25,0xae,
  0xcc,0x32,0xaf,0xcf,0x34,0xb2,0xcd,0x15,0xc1,0xdb,0x22,0xc5,0xd7,0x3c,0xd7,0xe1,
  0x6f,0x9a,0xb8,0x78,0x97,0xb4,0x40,0xa2,0xc3,0x64,0xb6,0xd1,0x45,0xd5,0xde,0x59,
  0xc3,0xda,0x48,0xdf,0xe9,0x7b,0xc5,0xde,0x79,0xcf,0xe2,0x71,0xd1,0xe0,0x80,0x9e,
  0xba,0x83,0xa7,0xc2,0x81,0xc9,0xdb,0x98,0xcf,0xe1,0x97,0xd5,0xe6,0xa9,0xcd,0xde,
  0xbc,0xde,0xeb,0xa8,0xe1,0xef,0xc9,0xeb,0xf4,0xdd,0xee,0xf6,0xe2,0xf2,0xf6,0xfd,
  0xfa,0xfd,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x21,0xf9,0x04,
  0x01,0x01,0x00,0x37,0x00,0x2c,0x00,0x00,0x00,0x00,0x16,0x00,0x16,0x00,0x00,0x06,
  0xfe,0xc0,0xdb,0xed,0x76,0xbb,0xdd,0x6e,0xb7,0xdb,0xed,0x76,0xbb,0xdd,0x6e,0xb7,
  0xdb,0xed,0x76,0xbb,0xdd,0x6e,0xb7,0xdb,0xed,0x76,0xbb,0xdd,0x6e,0xb7,0xdb,0xed,
  0x76,0xbb,0xdd,0x6e,0xb7,0xdb,0xed,0x76,0xbb,0xdd,0x6e,0xb7,0xdb,0xed,0x76,0xbb,
  0xdd,0x6e,0xb7,0xdb,0xed,0x76,0xbb,0xdd,0x80,0xb7,0xdb,0xed,0x76,0xbb,0xdd,0x6e,
  0xb7,0xdb,0xed,0x76,0xbb,0x11,0x02,0xb7,0xdb,0xed,0x76,0xbb,0xdd,0x6e,0xb7,0xdb,
  0xed,0x76,0xbb,0xdd,0x6e,0xb7,0xdb,0x8d,0x20,0x62,0x04,0x6e,0xb7,0xdb,0xed,0x76,
  0xbb,0xdd,0x6e,0xb7,0xdb,0xed,0x76,0xbb,0xdd,0x6e,0x04,0x16,0xcd,0xb2,0x10,0x00,
  0x6f,0xb7,0xdb,0xed,0x76,0xbb,0xdd,0x6e,0xb7,0xdb,0xed,0x76,0xbb,0x11,0x44,0xb6,
  0xd8,0xa7,0xb2,0x10,0xdc,0x6e,0xb7,0xdb,0xed,0x76,0xbb,0xdd,0x6e,0xb7,0xdb,0x8d,
  0xb0,0xb2,0xd1,0x52,0x1e,0x10,0x66,0x11,0xb8,0xdd,0x6e,0xb7,0xdb,0xed,0x76,0xbb,
  0xdd,0x6e,0x84,0x95,0xad,0xa6,0x02,0x62,0x28,0x9e,0x93,0x65,0x21,0xb8,0xdd,0x6e,
  0xb7,0xdb,0xed,0x76,0xbb,0x11,0x42,0x35,0x98,0xc9,0x42,0x89,0x5c,0x3c,0x20,0xc9,
  0x22,0x70,0xbb,0xdd,0x6e,0xb7,0xdb,0x8d,0x70,0x80,0x3c,0x1a,0x1b,0x4a,0x24,0x92,
  0x99,0x2c,0x0c,0x06,0x42,0xe0,0x76,0xbb,0xdd,0x6e,0x84,0x02,0x81,0x00,0x2c,0x10,
  0x34,0x1d,0x4a,0xe4,0x33,0x21,0x10,0x08,0x05,0x42,0xe0,0x76,0xbb,0xdd,0x6e,0xb7,
  0xdb,0xed,0x06,0x70,0xbd,0x3a,0x93,0x12,0x63,0x70,0xbb,0xdd,0x6e,0xb7,0xdb,0xb2,
  0xed,0x76,0xbb,0xdd,0x6e,0xb7,0x5b,0xa2,0x36,0x6b,0x49,0x4a,0x8c,0xc1,0xed,0x76,
  0xbb,0xdd,0x6e,0xb7,0xdb,0x0d,0x78,0xbb,0xdd,0x6e,0xb7,0x83,0x6d,0xe6,0x9a,0x9c,
  0x18,0x83,0xdb,0xed,0x76,0xbb,0xdd,0x6e,0xb7,0xdb,0xed,0x76,0xbb,0xdd,0x08,0x36,
  0x19,0x4a,0xf2,0x61,0x0c,0x6e,0xb7,0xdb,0xed,0x76,0xbb,0xdd,0x6e,0xb7,0xdb,0xed,
  0x76,0x23,0xcc,0x48,0x9d,0x49,0x46,0x31,0xb8,0xdd,0x6e,0xb7,0x1b,0xf0,0x76,0xbb,
  0xdd,0x6e,0xb7,0xdb,0xed,0x46,0x40,0x3c,0x14,0x0a,0x45,0x61,0x70,0xbb,0xdd,0x6e,
  0xb7,0xdb,0xed,0x76,0xbb,0xdd,0x6e,0xb7,0x1b,0x20,0x30,0x18,0x08,0x06,0x01,0xc0,
  0xed,0x76,0xbb,0xdd,0x6e,0xb7,0xdb,0xed,0x76,0xbb,0xdd,0x6e,0xb7,0xdb,0xed,0x76,
  0xbb,0xdd,0x6e,0x37,0xe0,0xed,0x76,0xbb,0xdd,0x6e,0xb7,0xdb,0xed,0x76,0xbb,0xdd,
  0x6e,0xb7,0xdb,0xed,0x76,0xbb,0xdd,0x6e,0xb7,0xdb,0xed,0x76,0xbb,0xdd,0x6e,0xb7,
  0xdb,0xed,0x76,0xbb,0xdd,0x6e,0xb7,0xdb,0xed,0x76,0xbb,0xdd,0x6e,0xb7,0xdb,0xed,
  0x16,0x04,0x00,0x3b
  };


class PollDialog;

class QicoWindow : public FXMainWindow {
  FXDECLARE(QicoWindow)

    protected:
  FXHorizontalFrame*   statusbarframe; // Frame for statusbars
  FXVerticalFrame*   contents;       // Frame for toolbar
  FXHorizontalFrame*   mainframe;      // Frame for "Main window"
  FXHorizontalFrame*     buttons;      // Frame for "Main window"
  FXHorizontalFrame*   toolbarcontainer;      // Frame for "Main window"
  FXVerticalFrame*     transmitframe;  // Frame for Send/Receive
  
  FXProgressBar*       sendbar;        // Send-file progressbar
  FXProgressBar*       send_t_bar;     // Send-total progressbar
  FXProgressBar*       recvbar;        // Recv-file progressbar
  FXProgressBar*       recv_t_bar;     // Recv-total progressbar
  
  FXStatusLine         *statusbar;      // Main status line
  FXStatusLine         *statusbar_;     // Secondary status line
  
  FXSplitter*          splitter;        // Splitter in "Main window"
  FXSplitter*          splitter_session;// Splitter in "Session"

  FXButton*            exitbutton;     // "Quit" button
  FXButton*            skipbutton;     // "Skip file" button
  FXButton*            rereadbutton;   // "Reread cfg" button
  FXButton*            rescanbutton;   // "Rescan" button
  FXButton*            pollbutton;     // "Poll" button
  FXButton*            sendbutton;     // "Poll" button
  FXButton*            freqbutton;     // "FReq" button
  FXButton*            nodelistbutton; // "Nodelist info" button

  FXList*              queuebox;       // Queue box

  FXText*              mainlog;        // Mainlog box
  FXText*              masterlog;      // Masterlog box

  FXToolBar*           toolbar;        // Toolbar
  FXToolBarTab*        toolbartab;     // Toolbartab

  FXGroupBox*          queuegroup;     // Group for queue
  FXGroupBox*          recvgroup;      // Group for Recv-file
  FXGroupBox*          sendgroup;      // Group for Send-file
  FXGroupBox*          infogroup;      // Group for EMSI info
  FXGroupBox*          maingroup;      // Group for Mainlog
  FXGroupBox*          mastergroup;    // Group for Masterlog

  FXTabBook*           tabbook;        // TabBook for tab1,tab2....
  FXTabItem*           tab1;           // "Main Window" tab
  FXTabItem*           tab2;           // "Session" tab

  FXButton*            switchbutton1;           // "Main Window" tab
  FXButton*            switchbutton2;           // "Session" tab
  FXButton*            switchbutton3;           // "Outbound manager" tab

  FXTable*             outboundtable;

  FXLabel*             queue_label;    // Label in queue
  FXLabel*             sendfile_label; // Label with send-file info
  FXLabel*             sendfile_total_label; // Label with send-total info
  FXLabel*             recvfile_label; // Label with recv-file info
  FXLabel*             recvfile_total_label;// Label with recv-tptal info

  FXLabel*             emsi_station_label; //Labels with EMSI
  FXLabel*             emsi_location_label;
  FXLabel*             emsi_sysop_label;
  FXLabel*             emsi_flags_label;
  FXLabel*             emsi_phone_label;
  FXLabel*             emsi_aka_label;
  FXLabel*             emsi_pwd_label;
  FXLabel*             emsi_lst_label;

  FXSwitcher*          switcher;

  FXIcon*              skipicon;          // Icons                            
  FXIcon*              exiticon;
  FXIcon*              rereadicon;
  FXIcon*              rescanicon;
  FXIcon*              pollicon;
  FXIcon*              nodelisticon;
  FXIcon*              freqicon;
  FXIcon*              connectingicon;
  FXIcon*              sendicon;

  PollDialog*          polldialog;
  FXHiliteStyle style;

protected:
  QicoWindow(){}
 public:
  enum {
    ID_MAIN_LOG=FXMainWindow::ID_LAST,
    ID_MASTER_LOG,
    ID_MAIN,
    ID_SESS,
    ID_OUTBOUND,
    ID_OUTBOUND_BOX,
    ID_TIMER,
    ID_POLL,
    ID_SEND,
    ID_NODELIST,
    ID_FREQ,
    ID_SKIP,
    ID_REREAD,
    ID_TOOLBAR_TAB,
    ID_RESCAN,
    ID_QUIT,
    ID_PANEL,
    ID_LAST
  };
public:
  long onCmdTimer(FXObject*,FXSelector,void*);
  long onCmdSkip(FXObject*,FXSelector,void*);
  long onCmdQuit(FXObject*,FXSelector,void*);
  long onCmdReread(FXObject*,FXSelector,void*);
  long onCmdRescan(FXObject*,FXSelector,void*);
  long onCmdPoll(FXObject*,FXSelector,void*);
  long onCmdSend(FXObject*,FXSelector,void*);
  long onCmdFreq(FXObject*,FXSelector,void*);
  long onCmdNodelist(FXObject*,FXSelector,void*);
  long onCmdMain(FXObject*,FXSelector,void*);
  long onCmdSess(FXObject*,FXSelector,void*);
  long onCmdOutbound(FXObject*,FXSelector,void*);
  long onCmdOutboundKeyPress(FXObject*,FXSelector,void* ptr);

public:
  QicoWindow(FXApp* a);
  void create();
  void rescanOutbound();
  void fresh_session_info();
  void freshpfile(int b, int e, pfile_t *s, int act);
  void wlog(char *fmt,...);
  void wlog1(char *fmt,...);
  void stylizeall();
};

class PollDialog : public FXDialogBox {
  FXDECLARE(PollDialog)
protected:
  FXHorizontalFrame* poll_mainframe;
  FXHorizontalFrame* poll_filesframe;
  FXHorizontalFrame* poll_toolbarcontainer;
  FXVerticalFrame*   poll_pollframe;
  FXVerticalFrame*   poll_flavourframe;
  FXButton*          poll_accept_button;
  FXButton*          poll_cancel_button;
  FXButton*          poll_browse_button;
  FXToolBar*         poll_toolbar;
  FXGroupBox*        poll_nodelistgroup;
  FXGroupBox*        poll_pollgroup;
  FXLabel*           poll_addresslabel;
  FXLabel*           poll_fileslabel;
  FXTextField*       poll_addressfield;
  FXTextField*       poll_filesfield;
  FXLabel*           poll_nodelistlabel;
  FXRadioButton*     poll_hold_flavour;
  FXRadioButton*     poll_normal_flavour;
  FXRadioButton*     poll_direct_flavour;
  FXRadioButton*     poll_crash_flavour;
  FXRadioButton*     poll_imm_flavour;
  QicoWindow*        qwin;
private:
  PollDialog(){}
 public:
  enum {
    ID_ADDRESS_ENTER=FXDialogBox::ID_LAST,
    ID_FILES_ENTER,
    ID_DO_POLL,
    ID_FILE_BROWSE,
    ID_FREQ_ENTER,
    ID_LAST
  };
 public:
  long onCmdAddrEnter(FXObject*,FXSelector,void* ptr);
  long onCmdFilesEntered(FXObject*,FXSelector,void*);
  long onCmdDoPoll(FXObject*,FXSelector,void*);
  long onCmdFileBrowse(FXObject*,FXSelector,void*);
  void show();
  PollDialog(QicoWindow* owner);
  virtual ~PollDialog();
};
